/******************************************************************************
 *
 * Copyright (c) 2000-2001 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: HwrUtilsLib.h
 *
 * Release: Palm OS 5.0 (133507)
 *
 * Description:
 *              Public API of HwrUtilsLib Library
 *
 * History:
 *
 *****************************************************************************/

#ifndef __HWRUTILSLIB_H__
#define	__HWRUTILSLIB_H__

#include <PalmTypes.h>


/********************************************************************
 * Constants
 ********************************************************************/
#define kHwrUtilsLibType		sysFileTLibrary
#define kHwrUtilsLibCreator		'HWut'
#define kHwrUtilsLibName		"HardwareUtils"	

// sysMakeROMVersion(major, minor, fix, stage, buildNum)
#define	kHwrUtilsLibVersionMajor		1
#define	kHwrUtilsLibVersionMinor		0
#define	kHwrUtilsLibVersionFix			0
#define	kHwrUtilsLibVersionStage		sysROMStageDevelopment
#define	kHwrUtilsLibVersionBuild 		0

// shared lib version
#define	kHwrUtilsLibAPIVersion 			sysMakeROMVersion(kHwrUtilsLibVersionMajor, kHwrUtilsLibVersionMinor, kHwrUtilsLibVersionFix, kHwrUtilsLibVersionStage, kHwrUtilsLibVersionBuild)

typedef enum
{
	kHwrUtilsLibGetMacAddr,
	kHwrUtilsLibSetMacAddr,
	kHwrUtilsLibGetDeviceId,
	kHwrUtilsLibSetDeviceId,
	
	
	kHwrUtilsLibSetRegisterAddr,
	
	
	kHwrUtilsLibGetRegisterValue,
	
	kHwrUtilsLibSetRegisterValue,
	
	kHwrUtilsLibGetUserInputPassword,
	kHwrUtilsLibSetUserInputPassword
	
	
} HwrUtilsLibCtlCodeType;


/********************************************************************
 * Error codes
 ********************************************************************/
#define hwrUtilsLibErrBase					appErrorClass
#define hwrUtilsLibErrVersion				(hwrUtilsLibErrBase + 1)
#define hwrUtilsLibErrParam					(hwrUtilsLibErrBase + 2)
#define hwrUtilsLibErrNotOpen				(hwrUtilsLibErrBase + 3)
#define hwrUtilsLibErrStillOpen				(hwrUtilsLibErrBase + 4)
#define hwrUtilsLibErrNotAvailable			(hwrUtilsLibErrBase + 5)
#define hwrUtilsLibErrNoMemory				(hwrUtilsLibErrBase + 6)
#define hwrUtilsLibErrUnknown				(hwrUtilsLibErrBase + 7)


/********************************************************************
 * Traps
 ********************************************************************/
#define kHwrUtilsLibTrapOpen					sysLibTrapOpen
#define kHwrUtilsLibTrapClose					sysLibTrapClose
#define kHwrUtilsLibTrapSleep					sysLibTrapSleep
#define kHwrUtilsLibTrapWake					sysLibTrapWake

#define kHwrUtilsLibTrapGetAPIVersion			(sysLibTrapCustom)
#define kHwrUtilsLibTrapControl					(sysLibTrapCustom + 1)
#define kHwrUtilsLibTrapGet5VOut				(sysLibTrapCustom + 2)
#define kHwrUtilsLibTrapSet5VOut				(sysLibTrapCustom + 3)
#define kHwrUtilsLibTrapGetKeyBacklight			(sysLibTrapCustom + 4)
#define kHwrUtilsLibTrapSetKeyBacklight			(sysLibTrapCustom + 5)
#define kHwrUtilsLibTrapGetBuzHiVol				(sysLibTrapCustom + 6)
#define kHwrUtilsLibTrapSetBuzHiVol				(sysLibTrapCustom + 7)
#define kHwrUtilsLibTrapSetVibrator				(sysLibTrapCustom + 8)
#define kHwrUtilsLibTrapSetRedLED				(sysLibTrapCustom + 9)
#define kHwrUtilsLibTrapSetGreenLED				(sysLibTrapCustom + 10)
#define kHwrUtilsLibTrapGetVibrator				(sysLibTrapCustom + 11)
#define kHwrUtilsLibTrapGetRedLED				(sysLibTrapCustom + 12)
#define kHwrUtilsLibTrapGetGreenLED				(sysLibTrapCustom + 13)
#define kHwrUtilsLibGetBTState					(sysLibTrapCustom + 14)
#define kHwrUtilsLibSetBTState					(sysLibTrapCustom + 15)
#define kHwrUtilsLibGetNumericKeyClickSnd		(sysLibTrapCustom + 16)
#define kHwrUtilsLibSetNumericKeyClickSnd		(sysLibTrapCustom + 17)


/********************************************************************
 * Prototypes
 ********************************************************************/
#ifdef BUILDING_HWRUTILSLIB
Err HwrUtilsLibOpen();
Err HwrUtilsLibClose();
Err HwrUtilsLibWake();
Err HwrUtilsLibSleep();

UInt32 HwrUtilsLibGetAPIVersion();

Err HwrUtilsLibControl(UInt16 controlCode, void *controlDataP, UInt16 *controlDataLenP);
UInt32 HwrUtilsLibGet5VOut();
void HwrUtilsLibSet5VOut(UInt32 value);
UInt32 HwrUtilsLibGetKeyBacklight();
void HwrUtilsLibSetKeyBacklight(UInt32 value);
UInt32 HwrUtilsLibGetBuzHiVol();
void HwrUtilsLibSetBuzHiVol(UInt32 value);

#else
Err HwrUtilsLibOpen(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapOpen);
Err HwrUtilsLibClose(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapClose);
Err HwrUtilsLibWake(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapWake);
Err HwrUtilsLibSleep(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapSleep);

UInt32 HwrUtilsLibGetAPIVersion(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetAPIVersion);

Err HwrUtilsLibControl(UInt16 refnum, UInt16 controlCode, void *controlDataP, UInt16 *controlDataLenP)
			SYS_TRAP(kHwrUtilsLibTrapControl);
UInt32 HwrUtilsLibGet5VOut(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGet5VOut);
void HwrUtilsLibSet5VOut(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSet5VOut);
UInt32 HwrUtilsLibGetKeyBacklight(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetKeyBacklight);
void HwrUtilsLibSetKeyBacklight(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSetKeyBacklight);
UInt32 HwrUtilsLibGetBuzHiVol(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetBuzHiVol);
void HwrUtilsLibSetBuzHiVol(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSetBuzHiVol);
UInt32 HwrUtilsLibGetVibrator(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetVibrator);
void HwrUtilsLibSetVibrator(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSetVibrator);
UInt32 HwrUtilsLibGetRedLED(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetRedLED);
void HwrUtilsLibSetRedLED(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSetRedLED);			
UInt32 HwrUtilsLibGetGreenLED(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibTrapGetGreenLED);						
void HwrUtilsLibSetGreenLED(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibTrapSetGreenLED);		
UInt32 HwrUtilsLibGetBTState(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibGetBTState);						
void HwrUtilsLibSetBTState(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibSetBTState);													
UInt32 HwrUtilsLibGetNumericKeyClickSnd(UInt16 refnum)
			SYS_TRAP(kHwrUtilsLibGetNumericKeyClickSnd);						
void HwrUtilsLibSetNumericKeyClickSnd(UInt16 refnum, UInt32 value)
			SYS_TRAP(kHwrUtilsLibSetNumericKeyClickSnd);																
			
#endif

#endif 	//__HWRUTILSLIB_H__
