#ifndef _WLAN_API_PRIV_H_
#define _WLAN_API_PRIV_H_


//**************************************************************************************************************
// PRIVATE API
//**************************************************************************************************************


/*----------------------------------------------------------------------------------------------------
 *                      Private NetIF settings for the WLAN network interface. 
 *
 * netIFSetting80211...  Datatype  Description
 * -----------------     --------  -------------------------------------------------------------------
 *
 *  PowerMode            UInt16     Read/write. Can be read/written at any time. When not connected,
 *                                  gets/sets the power mode in the preferences, to be used for future
 *                                  connections. When connected, gets the current power mode or attempts
 *                                  to switch to the desired power mode (but see below). Possible power 
 *                                  modes are:
 *  
 *                                      _80211_POWER_MODE_CAM       Full power
 *                                      _80211_POWER_MODE_FAST_PSP  Normal low-power mode
 *                                      _80211_POWER_MODE_MAX_PSP   Extra low-power mode
 *                                      _80211_POWER_MODE_SLEEP     MAC sleep mode (currently not used)
 *                                      _80211_POWER_MODE_OFF       MAC off (read only)
 *
 *                                  The application may attempt to set any of the top three modes. The
 *                                  two PSP modes differ in the wake up interval, with MAX_PSP waking
 *                                  less often, but suffering a greater throughput penalty. The default
 *                                  power mode is CAM. To switch to _80211_POWER_MODE_OFF, the interface
 *                                  must be brought down, usually by closing NetLib. Using either of the
 *                                  PSP modes will cause a large drop in the overall speed of the 
 *                                  connection. It is recommended that this call be used before the driver
 *                                  loads, as the effect of switching power modes on the fly has not been
 *                                  tested.
 *
 *----------------------------------------------------------------------------------------------------*/

// Power modes
#define _80211_POWER_MODE_CAM                   0       // full power
#define _80211_POWER_MODE_FAST_PSP              1       // normal power save (reduced network performance)
#define _80211_POWER_MODE_MAX_PSP               2       // advanced power save (further performance reduction)
#define _80211_POWER_MODE_SLEEP                 3       // read-only (currently not used)
#define _80211_POWER_MODE_OFF                   4       // read-only (probably won't see this)

/*====================================================================================
 * NetIFSettingGet/Set() private settings. 
 *====================================================================================*/
#define wlanNetIFSettingPrivateOffset           0x200

typedef enum
{
    netIFSettingDummy = netIFSettingCustom + wlanNetIFSettingPrivateOffset,
    reserved,                                   // reserved
    netIFSetting80211PowerMode,                 // Get/set power mode
} wlanNetIfPrivateSettings; 


#endif