/******************************************************************************
 *
 * 	Barcode Scanner Demo Application
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			ScanDemo.c
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Demo Application for Bar Code Scanner
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 
#ifdef __cplusplus
	extern "C" {
#endif

#include "PalmOS.h"					// all the system toolbox headers
#include <HwrMiscFlags.h>			// gives HwrBatteryLevel

#include "BabboChar.h"

#include "ScanMgrDef.h"				// scan manager constants
#include "ScanMgrStruct.h" 			// scan manager structures
#include "ScanMgr.h" 				// scan manager API functions

#include "ScanDemoRsc.h"			// application resource defines
#include "SetupDlgs.h"				// Setup dialogs for this app 
#include "Utils.h"					// Utilities for this app

#include <SysEvtMgr.h>

/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static void AppStart(void);
static void AppStop(void);
static Boolean MainFormHandleEvent(EventPtr event);
static void MainFormHandleMenu( UInt16 menuSel );
static void EventLoop(void);

static Boolean ShowDecodeData(void);
static void MainFormOnInit(void);
static void UpdateMainForm(void);
static Boolean ApplicationHandleEvent (EventPtr event);

/***********************************************************************
 * Global Variables
 **********************************************************************/
static 		Boolean g_bScanManagerOpen = false;
static		Boolean g_bAutoScan = false;
static 		Int32 g_time;
static 		MenuBarPtr g_CurrentMenu;

Boolean 	g_bExtendedData;
MemHandle	g_hExtendedData;
Int16		g_ExtendedDataLength;
Int16 		g_ExtendedDataType;
UInt8		*g_pExtendedData;
extern Boolean 	g_bEnabledAll;



#ifdef __cplusplus
	}
#endif


/***********************************************************************
 *
 * FUNCTION:     	AppStart
 *
 * DESCRIPTION: 	1. Check if a Barcode device is connected
 * 					2. If Success, open the Decode Driver Library
 * 					3. If Success, enable Scanninghardware.
 * 					4. Enter the main application form
 *
 * PARAMETERS:   	None.
 *
 * RETURNED:     	Nothing.
 *
 ***********************************************************************/
static void AppStart(void)
{
	Err error;	
	UInt32 HwrUtilsVersion=0;

	//Check if the BarCode Scanner Device is connected.
	if (ScnIsJanamUnit())
	{
		error = ScnOpenDecoder();	//Open the scanner library
		if (!error)
		{
			g_bScanManagerOpen = true; 	// set global variable that all is well
			ScnCmdScanEnable();	// Enable scanning
		}
		else
		{
			if (error == scanMgrErrLowBatt)
				FrmAlert( BatteryErrorAlert );
			else
				FrmAlert( OpenFailedAlert );
		}
	}
	else	
		FrmAlert( WrongHardwareAlert );
	
	// Initialize and draw the main form.
	FrmGotoForm( MainForm );
}


/***********************************************************************
 *
 * FUNCTION: 		AppStop
 *
 * DESCRIPTION: 	1. Check if the Scanner is open properly
 * 					2. Disable Scanning
 *					3. Close the Scan driver library
 * 					
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void AppStop(void)
{
	UInt16		cardNo;
	LocalID		dbID;
	
	// check to be sure that we earlier opened the decoder successfully 
	if (g_bScanManagerOpen)
	{
		// Disable the scanner, and then close the decoder.
		ScnCmdScanDisable();
		
		// Close the decoded
		ScnCloseDecoder();
		
		SysCurAppDatabase (&cardNo, &dbID);
		AlmSetAlarm (cardNo, dbID, 0, 0, 0);		// Clear any alarm
	}
}


/***********************************************************************
 *
 * FUNCTION:		MainFormHandleEvent
 *
 * DESCRIPTION:		Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr event)
{
	Boolean		bHandled = false;
	Int16		ret;
	UInt16		cardNo;
	LocalID		dbID;
	UInt16	extendedDataFlag;

	switch( event->eType )
	{
		case frmOpenEvent:
			MainFormOnInit();
			bHandled = true;
			break;
			
		case frmUpdateEvent:
			UpdateMainForm();
			break;
			
		case menuEvent:
			MainFormHandleMenu(event->data.menu.itemID);
			bHandled = true;
			break;

		case scanDecodeEvent:
			//A Scan Decode Event has performed
			extendedDataFlag = ((ScanEventPtr)event)->scanData.scanGen.data1;
			g_ExtendedDataLength  = (Int16)(((ScanEventPtr)event)->scanData.scanGen.data2);
			
			//Check if there is extended flag 
			if (extendedDataFlag & EXTENDED_DATA_FLAG)
				g_bExtendedData = true;
			else
				g_bExtendedData = false;
				
			//Show the decode data
			ShowDecodeData();

			if (g_bAutoScan == true)
			{
				SysCurAppDatabase (&cardNo, &dbID);
				AlmSetAlarm (cardNo, dbID, 0, (TimGetSeconds()+g_time), 0);
			} 
		
			bHandled = true;
			break;
								
		case popSelectEvent:
		{
			if (event->data.popSelect.listID == MainBeepModeListList)
			{
				Int16 nBeepMode = event->data.popSelect.selection;

				//Enable Beep after success decode
				ScnSetBeepAfterGoodDecode(nBeepMode);
				ScnCmdSendParams(No_Beep);
			}
			else if (event->data.popSelect.listID == MainLightModeList)
			{
				LightsMode_t nLightMode = (LightsMode_t) event->data.popSelect.selection;
				
				ScnCmdGetSetLightsMode(true, QT_CURRENT, (UInt8*) &nLightMode); 
				//ScnCmdSendParams(No_Beep);
				//Don't need ScnCmdSendParams here.
				//Extended function will take effect immediately
			}
			else if (event->data.popSelect.listID == MainScanModeList)
			{
				UInt16 ScanMode=0;
				if (event->data.popSelect.selection == 0)
					ScanMode = LEVEL;
				else	
					ScanMode = PULSE;
				ScnSetTriggeringModes(ScanMode);
				ScnCmdSendParams(No_Beep);
				UpdateMainForm();
				bHandled = true;				
			}	
			break;
			
			
		}

		case ctlSelectEvent:
		{
			switch(event->data.ctlEnter.controlID)
			{
				// Scan Button
				case MainSCANButton:
					ret = ScnCmdStartDecode();
				  	bHandled = true;
	   				break;
	   				
				// Stop Button
				case MainStopButton:
					ScnCmdStopDecode();
				  	bHandled = true;
	   				break;
	   				
			}  //end switch
		}
		
		
	}//end if

	return(bHandled);
}


/***********************************************************************
 *
 * FUNCTION:		MainFormOnInit
 *
 * DESCRIPTION: 	Initialize the main form (in response to a frmOpenEvent)
 * 					Calls "UpdateMainForm" to initialize all the controls on
 * 					the main form.  Then it draws the form (FrmDrawForm).
 *
 * PARAMETERS:		None
 *
 * RETURNED:		Nothing
 *
 ***********************************************************************/
static void MainFormOnInit()
{
	FormPtr pForm = FrmGetActiveForm();
	if( pForm )
	{
		UpdateMainForm();
		FrmDrawForm( pForm );
	}
}


/***********************************************************************
 *
 * FUNCTION:		UpdateMainForm
 *
 * DESCRIPTION: 	Update the main form screen
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
void UpdateMainForm(void)
{
	LightsMode_t nLightMode;
	UInt16 ScanMode;
	ControlPtr pCtl;
	Char *  label;
	
	ListPtr pList = (ListPtr) GetObjectPtr( MainBeepModeListList );
	LstSetSelection(pList, ScnGetBeepAfterGoodDecode());
	label = LstGetSelectionText (pList, ScnGetBeepAfterGoodDecode());
	pCtl = (ControlPtr) GetObjectPtr (MainBeepModePopTriggerPopTrigger);
	CtlSetLabel (pCtl, label);
		
	ScnCmdGetSetLightsMode(false, QT_CURRENT, (UInt8*) &nLightMode); 
	pList = (ListPtr) GetObjectPtr( MainLightModeList );
	LstSetSelection(pList, nLightMode);	
	label = LstGetSelectionText (pList, nLightMode);
	pCtl = (ControlPtr) GetObjectPtr (MainLightModePopTriggerPopTrigger);
	CtlSetLabel (pCtl, label);

	pList = (ListPtr) GetObjectPtr( MainScanModeList );
	switch (ScnGetTriggeringModes())
	{
		case LEVEL:
			ScanMode = 0;
			break;
		case PULSE:
			ScanMode = 1;
			break;		
		default:			
			ScanMode = 0;
	}		
	LstSetSelection(pList, ScanMode);	
	label = LstGetSelectionText (pList, ScanMode);
	pCtl = (ControlPtr) GetObjectPtr (MainScanModePopTrigger);
	CtlSetLabel (pCtl, label);	
	
	// initialize the barcode type and barcode data fields
	SetFieldText( MainBarTypeField, "No Data", 20, false );
	SetFieldText( MainScandataField, "No Data", MAXBARCODELENGTH, false );
			
}


/***********************************************************************
 *
 * FUNCTION:		MainFormHandleMenu
 *
 * DESCRIPTION: 	Handles event received by main form
 * 
 * 					Called by MainFormHandleEvent for any menuEvent received.
 *
 * PARAMETERS:		menuSel - the menu item that was selected
 *
 * RETURNED:		Nothing
 *
 ***********************************************************************/
void MainFormHandleMenu(UInt16 menuSel)
{
	switch(menuSel)
	{
		// Options menu
		case OptionsResetDefaults:
			{
				LightsMode_t nLightMode = LM_ILLUM_AIMER;

				if (ScnIsJanamUnit()){
					ScnCmdScanDisable(); 			
					if(ScnCmdParamDefaults() == STATUS_OK)
						ScnCmdScanEnable(); 					// enable scanning
				}
				
				ScnCmdGetSetLightsMode(true, QT_CURRENT, (UInt8*) &nLightMode);
				ScnSetBeepAfterGoodDecode(ENABLE);
				ScnSetTriggeringModes(LEVEL);	
				ScnCmdSendParams(No_Beep);
				g_bEnabledAll = false;
			}
			FrmUpdateForm( MainForm, frmRedrawUpdateCode); 	// force repaint of form
			break;
		
		// About menu
		case OptionsAbout:
			OnAbout();
			break;
		
		// OCR Config menu
		case OCROCRConfig:
			OnOCRSetup();
			break;

		// Hardware Test menu
		case SetupHardwareTest:
			OnHardwareSetup();
			break;
		
		// Symbology Config menu	
		case SetupSymbologyConfig:
			OnSymbologySetup();
			break;
	}
}


/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:		A simple loop that obtains events from the Event
 *					Manager and passes them on to various applications and
 *					system event handlers before passing them on to
 *					FrmHandleEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	static EventType	event;
	UInt16 error;
	do
	{
		// Get the next available event.
		EvtGetEvent(&event, evtWaitForever);

		// Give the system a chance to handle the event.
	  	if( !SysHandleEvent (&event))
			if( !MenuHandleEvent (g_CurrentMenu, &event, &error))
				// Give the application a chance to handle the event.
				if( !ApplicationHandleEvent(&event) )
					// Let the form object provide default handling of the event.
 					FrmDispatchEvent(&event);
	} 
	while (event.eType != appStopEvent);
}



/***********************************************************************
 *
 * FUNCTION:		ApplicationHandleEvent
 *
 * DESCRIPTION:		An event handler routine that handles events at the application
 * 					level, particularly frmLoadEvents which control the GUI flow by 
 * 					setting the appropriate form handler into effect.  Also handles 
 * 					the scanBatteryEvent, should it happen anywhere in the app.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent (EventPtr event)
{
	UInt16 formID;
	FormPtr frm;
	
	if (event->eType == scanBatteryErrorEvent)
	{
		//Scandemo will not handle the battery event
		return (false);
	}
	
	else if (event->eType == frmLoadEvent)
	{
		// Load the form resource.
		formID = event->data.frmLoad.formID;
		frm = FrmInitForm (formID);
		FrmSetActiveForm (frm);		
		
		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmDispatchEvent each g_time is receives an
		// event.
		switch (formID)
		{
			case MainForm:
				FrmSetEventHandler (frm, MainFormHandleEvent);
				break;
		
			case HardwareForm:
				FrmSetEventHandler (frm, HardwareSetupHandleEvent);
				break;
	
			case SymbologyForm:
				FrmSetEventHandler (frm, SymbologyConfigSetupHandleEvent);
				break;

			case OCRForm:
				FrmSetEventHandler (frm, OCRSetupHandleEvent );
				break;			
			
			case AboutForm:
				FrmSetEventHandler (frm, AboutHandleEvent );
				break;
				
			default:
				break;
		}
		return (true);
	}
	return (false);
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:		This function is the equivalent of a main() function
 *					in standard C.  It is called by the Emulator to begin
 *					execution of this application.
 *
 * 					We call AppStart to do any initial processing.
 * 					If the decoder was successfully initialized, we proceed
 * 					with the application.  Otherwise we exit right away, 
 * 					after calling AppStop to do its cleanup job.
 * 					
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *					cmdPBP - parameter block for the command.
 *					launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error code.
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	EventType	scanEvent;
	
	// Check for a normal launch.
	if (cmd == sysAppLaunchCmdNormalLaunch)
	{
		// Open the decoder and specify our initial (main) form.
		AppStart();

		// only continue if the decoder was successfully opened
		if (g_bScanManagerOpen)		
			EventLoop(); // Start up the event loop.
			
		AppStop();
	}
	
	else if (cmd == sysAppLaunchCmdAlarmTriggered && g_bAutoScan == true)
	{
		scanEvent.eType = ctlSelectEvent;
		scanEvent.data.ctlEnter.controlID = MainSCANButton;
		EvtAddEventToQueue(&scanEvent);
	}
	return(0);
}

/***********************************************************************
 *
 * FUNCTION: 		OnDecoderEvent
 *
 * DESCRIPTION:		Called when received decodeevent
 *					Update the mainform with the received barcode data
 * 					
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
Boolean ShowDecodeData()
{
	static Char BarTypeStr[300]=" ";
	MESSAGE 	decodeDataMsg;
	Int16 status;
	
	
	if (g_bExtendedData == true){
		g_hExtendedData = MemHandleNew(g_ExtendedDataLength+1);
		g_pExtendedData = (UInt8 *) MemHandleLock( g_hExtendedData );
		status = ScnGetExtendedDecodedData(g_ExtendedDataLength, &g_ExtendedDataType, g_pExtendedData);
		g_pExtendedData[g_ExtendedDataLength] = '\0';		
	}
	else{
		status = ScnGetDecodedData( &decodeDataMsg );
		g_ExtendedDataType = decodeDataMsg.type;
		g_ExtendedDataLength = decodeDataMsg.length;
		g_hExtendedData = MemHandleNew(g_ExtendedDataLength+1);
		g_pExtendedData = (UInt8 *) MemHandleLock( g_hExtendedData );
		g_pExtendedData[g_ExtendedDataLength] = '\0';
		MemMove( &g_pExtendedData[0], &decodeDataMsg.data[0], g_ExtendedDataLength+1 );		
	}

	if( status == STATUS_OK ) // if we successfully got the decode data from the API...
	{
		
		if( g_ExtendedDataType == 0){
			SetFieldText(MainBarTypeField, "No Scan", 30, true);
			SetFieldText(MainScandataField, "No Scan", 30, true);
			MemHandleUnlock(g_hExtendedData);
			MemHandleFree(g_hExtendedData);
		}
		else{
		
			ScnGetBarTypeStr(g_ExtendedDataType, BarTypeStr, 30);	
			SetFieldText( MainBarTypeField, BarTypeStr, 30, true);
			
			// Place the barcode data into the field and display
			SetFieldText(MainScandataField, (char *)&g_pExtendedData[0], g_ExtendedDataLength+1, true);
			MemHandleUnlock(g_hExtendedData);
			MemHandleFree(g_hExtendedData);
		}
	}
	else
			DbgMessage ("Decode Status Error\n");

	return(0);	    			
}




