/******************************************************************************
 *
 * 	Barcode Scanner Demo Application
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			SetupDlgs.c
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Demo Application for Bar Code Scanner
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 

#ifdef __cplusplus
	extern "C" {
#endif

#include <PalmOS.h>
#include "SetupDlgs.h"


#include "JanamScnCompatible.h"
#include "ScanDemoRsc.h" 		// include our project's resource ids
#include "ScanMgrStruct.h" 		// include scan manager structure defs
#include "ScanMgrDef.h" 		// include scan manager constant defs
#include "ScanMgr.h" 			// include scan manager API function defs
#include "Utils.h" 				// include utility functions used throughout
#include "HwrUtilsLib.h"		// Babbo Hwr Utility functions


static void HardwareSendToDecoder();
static Err HwrUtilsLib_OpenLibrary(UInt16 *refNumP);
static Err HwrUtilsLib_CloseLibrary(UInt16 refNum);

#ifdef __cplusplus
	}
#endif

/***********************************************************************
 ***********************************************************************
	Global Variables
 ***********************************************************************   
 ***********************************************************************/
UInt16 		g_HwrUtilsLibRefNum		=0;
Int16  		g_nSel					=noListSelection;
Boolean 	g_bEnabledAll			=false;
Boolean 	g_HwrUtilsVersionCheck	=false;
Boolean		g_KeyBacklight			=false;


/***********************************************************************
 ***********************************************************************
	Define for the Symbology Menu and Options
 ***********************************************************************   
 ***********************************************************************/

static const struct symFlagTblEnt {
	char label[16];
	UInt32 flagBit;
} symFlagTbl[] = {
	{ "Check Enable", 		SYMBOLOGY_CHECK_ENABLE},
	{ "Check Xmit", 		SYMBOLOGY_CHECK_TRANSMIT},
	{ "Start/Stop Xmit", 	SYMBOLOGY_START_STOP_XMIT},
	{ "Append Mode",		SYMBOLOGY_ENABLE_APPEND_MODE},
	{ "Full ASCII", 		SYMBOLOGY_ENABLE_FULLASCII},
	{ "Concatenate", 		SYMBOLOGY_CODABAR_CONCATENATE},
	{ "2 Check Digits", 	SYMBOLOGY_CHECK_ENABLE},
	{ "Old Style", 			SYMBOLOGY_TELEPEN_OLD_STYLE},
	{ "Expanded", 			SYMBOLOGY_EXPANDED_UPCE},
	{ "Number Sys Xmit", 	SYMBOLOGY_NUM_SYS_TRANSMIT},
	{ "2 Dig Addenda", 		SYMBOLOGY_2_DIGIT_ADDENDA},
	{ "5 Dig Addenda", 		SYMBOLOGY_5_DIGIT_ADDENDA},
	{ "Addenda Req", 		SYMBOLOGY_ADDENDA_REQUIRED},
	{ "Addenda Sep", 		SYMBOLOGY_ADDENDA_SEPARATOR},
	{ "Aztec Runes", 		SYMBOLOGY_AZTEC_RUNE},
	{ "Enable UPC/EAN", 	SYMBOLOGY_COMPOSITE_UPC},
	{ "Concat Req", 		SYMBOLOGY_CODABAR_CONCAT_REQ},
	{ "ISBT Concat", 		SYMBOLOGY_ISBT_CONCATENATE},
	{ "Code 32 conv", 		SYMBOLOGY_ENABLE_BASE32},
	{ "Convert ISBN", 		SYMBOLOGY_EAN13_ISBN_XLATE},
};

static struct symConfigTblEnt {
	UInt32 ofs;
	UInt32 txtTblBits;
	UInt8 symID;
} symConfigTbl[] =
{
	{ OFS(codabar),	F_CHK_ENA|F_CHK_XMIT|F_STRT_STP|F_CONCAT|F_CONCATREQ|F_MINMAX, barCODABAR },
	{ OFS(code39),	F_CHK_ENA|F_CHK_XMIT|F_STRT_STP|F_APPEND|F_FULLASC|F_BASE32|F_MINMAX, barCODE39 },
	{ OFS(int2of5),	F_CHK_ENA|F_CHK_XMIT|F_MINMAX, barI2OF5 },
	{ OFS(code93),	F_MINMAX, barCODE93 },
	{ OFS(code2of5),F_MINMAX, barD25 },
	{ OFS(iata25),	F_MINMAX, barIATA25 },
	{ OFS(matrix25),F_MINMAX, barMATRIX25 },
	{ OFS(code11),	F_2CHKDIG|F_MINMAX, barCODE11 },
	{ OFS(code128),	F_MINMAX|F_ISBTCONCAT, barCODE128 },
	{ OFS(telepen),	F_OLDSTYLE|F_MINMAX, barTELEPEN },
	{ OFS(upcA),	F_CHK_XMIT|F_NUMSYSXMIT|F_2DIGADDA|F_5DIGADDA|F_ADDAREQ|F_ADDASEP, barUPCA },
	{ OFS(couponCode), 0, barCOUPON },
	{ OFS(upcE),	F_CHK_XMIT|F_EXPAND|F_NUMSYSXMIT|F_2DIGADDA|F_5DIGADDA|F_ADDAREQ|F_ADDASEP, barUPCE },
	{ OFS(upcE),	0, barUPCE1 },
	{ OFS(ean13),	F_CHK_XMIT|F_ADDAREQ|F_ADDASEP|F_2DIGADDA|F_5DIGADDA|F_ISBN, barBOOKLAND_EAN },
	{ OFS(ean8),	F_CHK_XMIT|F_ADDAREQ|F_ADDASEP|F_2DIGADDA|F_5DIGADDA, barEAN8 },
	{ OFS(msi),		F_CHK_XMIT|F_MINMAX, barMSI },
	{ OFS(plesseyCode), F_MINMAX, barPlessey },
	{ OFS(rss),     0, barRSS },
	{ OFS(rssLim),  0, barRSS_LIM },
	{ OFS(rssExp),  F_MINMAX, barRSS_EXP },
	{ OFS(posiCode),F_MINMAX, barPOSICODE },
	{ OFS(triopticCode),0, barTRIOPTIC39 },
	{ OFS(codablock),F_MINMAX, barCODABLOCK },
	{ OFS(code16k),	F_MINMAX, barCODE16K },
	{ OFS(code49),	F_MINMAX,	barCODE49 },
	{ OFS(pdf417),	F_MINMAX,	barPDF417 },
	{ OFS(microPDF),F_MINMAX,	barMICROPDF },
	{ OFS(composite),F_MINMAX,	barCOMPOSITE },
	{ OFS(tlCode39),0,	barTLCODE39 },
	{ OFS(usps4Post),0,	barUSPS4CB },
	{ OFS(idtagPost),0, barIDTAG },
	{ OFS(usPostnet),F_CHK_XMIT, barPOSTNET },
	{ OFS(usPlanet),F_CHK_XMIT, barPLANET },
	{ OFS(britishPost),0, barBPO },
	{ OFS(canadaPost),0, barCANPOST },
	{ OFS(dutchPost), 0, barDUTCHPOST },
	{ OFS(australiaPost), 0, barAUSPOST },
	{ OFS(japanPost), 0, barJAPOST },
	{ OFS(chinaPost), F_MINMAX, barCHINAPOST },
	{ OFS(koreaPost), F_MINMAX, barKOREAPOST },
	{ OFS(qr),		F_MINMAX, barQR },
	{ OFS(datamatrix),F_MINMAX, barDATAMAXTRIX },
	{ OFS(maxicode),	F_MINMAX, barMAXICODE },
	{ OFS(aztec),	F_AZTECRUNE|F_MINMAX, barAZTEC }
};

/***********************************************************************
 ***********************************************************************
	Functions to convert List Item to Sym Id
 ***********************************************************************   
 ***********************************************************************/
tagBarType PrvBarCodeLstConversion(UInt32 LstNumber);
void HardwareStopAllTests();

 
/***********************************************************************
 *
 * FUNCTION: 		OnHardwareSetup
 *
 * DESCRIPTION: 	Sets HardwareForm the active form.
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
void OnHardwareSetup()
{
	FrmGotoForm( HardwareForm );
}

/***********************************************************************
 *
 * FUNCTION: 		HardwareFormOnInit
 *
 * DESCRIPTION: 	Initilize the Hardware Test Form
 *
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void HardwareFormOnInit()
{
	UInt32 HwrUtilsVersion =0;
	FormPtr pForm = FrmGetActiveForm();	
	FrmDrawForm( pForm );	
		
	g_HwrUtilsVersionCheck = false;
	if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
	{
		HwrUtilsVersion = HwrUtilsLibGetAPIVersion(g_HwrUtilsLibRefNum);
		HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
	}
	
	if (HwrUtilsVersion >= 1)
		g_HwrUtilsVersionCheck = true;
		
	if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
	{
		g_KeyBacklight = HwrUtilsLibGetKeyBacklight(g_HwrUtilsLibRefNum);
		HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
	}
	
	{
		ControlPtr pCtl;
		pCtl = (ControlPtr) GetObjectPtr(HardwareKeypadBacklightCheckbox);
		CtlSetValue(pCtl, g_KeyBacklight);
	}
}

/***********************************************************************
 *
 * FUNCTION: 		HardwareStopAllTests
 *
 * DESCRIPTION: 	Stop all Bar Code Scanner Hardware Tests
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
void HardwareStopAllTests()
{
	ScnCmdAimerOn(false);
	ScnCmdLightsOn(false);	
	
	//For safety. Check if the version support the following functions.
	if (g_HwrUtilsVersionCheck)
	{
		if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
		{
			HwrUtilsLibSetVibrator(g_HwrUtilsLibRefNum, false);
			HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
		}
		if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
		{
			HwrUtilsLibSetRedLED(g_HwrUtilsLibRefNum, false);
			HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
		}
		if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
		{
			HwrUtilsLibSetGreenLED(g_HwrUtilsLibRefNum, false);
			HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
		}
		if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
		{
			HwrUtilsLibSetKeyBacklight(g_HwrUtilsLibRefNum, g_KeyBacklight);
			HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
		}

	}
	ScnCmdBeep(No_Beep);	
}


/***********************************************************************
 *
 * FUNCTION: 		HardwareSetupHandleEvent
 *
 * DESCRIPTION: 	Event handler for the Hardware Setup form.
 * 					Handles the following events: 
 * 						frmOpenEvent and frmCloseEvent, 
 * 						ctlSelectEvent (for the OK and Cancel), 
 * 
 * PARAMETERS: 		ev - pointer to the event information structure
 * 
 * RETURNED: 		true  - we handled the event
 * 					false - not handled here, so pass it to next handler
 *
 ***********************************************************************/
Boolean HardwareSetupHandleEvent( EventPtr ev )
{
	Boolean bHandled = false;
	
	switch( ev->eType )
	{
		case ctlSelectEvent:
			if( ev->data.ctlSelect.controlID == HardwareCancelButton )
			{
				HardwareStopAllTests();	
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			else if( ev->data.ctlSelect.controlID == HardwareOKButton )
			{
				HardwareStopAllTests();				
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			
			//Aimer Light Test
			else if( ev->data.ctlSelect.controlID == HardwareAimerCheckbox)
			{
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(HardwareAimerCheckbox);
				if (CtlGetValue(pCtl))
					ScnCmdAimerOn(true);
				else
					ScnCmdAimerOn(false);
			}
			
			//Illum Light Test
			else if( ev->data.ctlSelect.controlID == HardwareIllumCheckbox)
			{
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(HardwareIllumCheckbox);
				if (CtlGetValue(pCtl))
					ScnCmdLightsOn(true);
				else
					ScnCmdLightsOn(false);				
			}

			//Vibrator Test
			else if( ev->data.ctlSelect.controlID == HardwareVibratorCheckbox)
			{
				//Version Check. Preliminary version do not have the following functions supported
				if (g_HwrUtilsVersionCheck)	
				{
					ControlPtr pCtl;
					pCtl = (ControlPtr) GetObjectPtr(HardwareVibratorCheckbox);
					if (CtlGetValue(pCtl))
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetVibrator(g_HwrUtilsLibRefNum, true);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}
					}
					else
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetVibrator(g_HwrUtilsLibRefNum, false);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}
					}
				}
			}
			
			//Red LED Test			
			else if( ev->data.ctlSelect.controlID == HardwareRedLEDCheckbox)
			{
				if (g_HwrUtilsVersionCheck)
				{
					ControlPtr pCtl;
					pCtl = (ControlPtr) GetObjectPtr(HardwareRedLEDCheckbox);
					if (CtlGetValue(pCtl))
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetRedLED(g_HwrUtilsLibRefNum, true);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}
					}
					else
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetRedLED(g_HwrUtilsLibRefNum, false);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}
					}
				}
			}
			
			//Green LED Test
			else if( ev->data.ctlSelect.controlID == HardwareGreenLEDCheckbox)
			{
				if (g_HwrUtilsVersionCheck)
				{
					ControlPtr pCtl;
					pCtl = (ControlPtr) GetObjectPtr(HardwareGreenLEDCheckbox);
					if (CtlGetValue(pCtl))
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetGreenLED(g_HwrUtilsLibRefNum, true);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}
					}
					else
					{
						if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
						{
							HwrUtilsLibSetGreenLED(g_HwrUtilsLibRefNum, false);
							HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
						}

					}
				}
			}
			
			//Beeper Test
			else if( ev->data.ctlSelect.controlID == HardwareBeeperCheckbox)
			{
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(HardwareBeeperCheckbox);
				if (CtlGetValue(pCtl))
				{
					ScnCmdBeep(Four_Short_High);	
				}
				else
				{
					ScnCmdBeep(No_Beep);	
				}
			}
			
			//Keypad Backlight Test
			else if( ev->data.ctlSelect.controlID == HardwareKeypadBacklightCheckbox)
			{
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(HardwareKeypadBacklightCheckbox);
				if (CtlGetValue(pCtl))
				{
					if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
					{
						HwrUtilsLibSetKeyBacklight(g_HwrUtilsLibRefNum, true);
						HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
					}
				}
				else
				{
					if (!HwrUtilsLib_OpenLibrary(&g_HwrUtilsLibRefNum))
					{
						HwrUtilsLibSetKeyBacklight(g_HwrUtilsLibRefNum, false);
						HwrUtilsLib_CloseLibrary(g_HwrUtilsLibRefNum);
					}

				}
			}
			break;

		case frmOpenEvent:
			HardwareFormOnInit();
			bHandled = true;
			break;
			
		
		case frmCloseEvent:
			break;
			
		default:
			break;
	}
	
	return bHandled;
}



/***********************************************************************
 ***********************************************************************
   Symbology Setup Dialog
 ***********************************************************************
 ***********************************************************************/
 
/***********************************************************************
 *
 * FUNCTION: 		OnSymbologySetup
 *
 * DESCRIPTION: 	Makes SymbologyForm the active form
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
void OnSymbologySetup()
{
	FrmGotoForm( SymbologyForm );
}


/***********************************************************************
 *
 * FUNCTION: 		PrvSymbolConvert
 *
 * DESCRIPTION: 	Helper function for Symbology Menu
 *					Convert Symbology Option to flag bit
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/

static UInt32 PrvSymbolConvert(UInt32 i)
{
	switch(i)
	{
		case 0:	return F_CHK_ENA;		
		case 1:	return F_CHK_XMIT;
		case 2:	return F_STRT_STP;
		case 3:	return F_APPEND;
		case 4:	return F_FULLASC;
		case 5:	return F_CONCAT;
		case 6:	return F_2CHKDIG;
		case 7:	return F_OLDSTYLE;
		case 8:	return F_EXPAND;
		case 9:	return F_NUMSYSXMIT;
		case 10:return F_2DIGADDA;
		case 11:return F_5DIGADDA;
		case 12:return F_ADDAREQ;
		case 13:return F_ADDASEP;
		case 14:return F_AZTECRUNE;
		case 15:return F_UPCCOMP;
		case 16:return F_CONCATREQ;
		case 17:return F_ISBTCONCAT;
		case 18:return F_BASE32;
		case 19:return F_ISBN;	
		case 20:return F_MINMAX;
	}
	return 0xFF;
}	


/***********************************************************************
 *
 * FUNCTION: 		SymbologyConfigSetControls
 *
 * DESCRIPTION: 	Send the Symbology setting to ScanCmd Library
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void SymbologyConfigSetControls(int sel)
{
	FormType *frmP = FrmGetActiveForm();
	unsigned i, ckn;//, ndx;
	struct symConfigTblEnt *symConfigEnt;
	SymFlagsRange_t symSetRange;
	Boolean bEnabled = false;
	UInt32 testbit=0;
	ControlType *ckP[8];
	
	ckP[0] = (ControlPtr) GetObjectPtr ( SymbologyCheck0Checkbox );
	ckP[1] = (ControlPtr) GetObjectPtr ( SymbologyCheck1Checkbox );
	ckP[2] = (ControlPtr) GetObjectPtr ( SymbologyCheck2Checkbox );
	ckP[3] = (ControlPtr) GetObjectPtr ( SymbologyCheck3Checkbox );
	ckP[4] = (ControlPtr) GetObjectPtr ( SymbologyCheck4Checkbox );
	ckP[5] = (ControlPtr) GetObjectPtr ( SymbologyCheck5Checkbox );
	ckP[6] = (ControlPtr) GetObjectPtr ( SymbologyCheck6Checkbox );
	ckP[7] = (ControlPtr) GetObjectPtr ( SymbologyCheck7Checkbox );
	
	
	
	//Read the Symbology setting for current selection
	symConfigEnt = &symConfigTbl[sel];
	if (symConfigEnt->txtTblBits & F_MINMAX)
	{
		symSetRange.dwStructSize = sizeof(SymFlagsRange_t);
	}
	else
		symSetRange.dwStructSize = sizeof(SymFlagsOnly_t);
	
	symSetRange.dwFlags = 0;
	symSetRange.dwMask = SYM_MASK_ALL;
	symConfigEnt = &symConfigTbl[sel];
	
	//add to chk if enabled
	ScnCmdReadSymbologyConfig(QT_CURRENT, symConfigEnt->symID, &symSetRange);
	
	if (symConfigEnt->txtTblBits & F_MINMAX)
	{
		MemHandle 		fldHandle;
		FieldType *fldP;
		char *preText;	
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMinFldField );
		fldHandle = FldGetTextHandle(fldP);
		preText = (Char * ) MemHandleLock(fldHandle);		
		symSetRange.dwMinLen = StrAToI(preText);
		MemHandleUnlock(fldHandle);	
		
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMaxFldField );
		fldHandle = FldGetTextHandle(fldP);
		preText = (Char * ) MemHandleLock(fldHandle);		
		symSetRange.dwMaxLen = StrAToI(preText);
		MemHandleUnlock(fldHandle);	
		
		//fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMaxFldField );
		//symSetRange.dwMaxLen = StrAToI((char*)FldGetTextHandle(fldP));
		
	
	}

	ckn=0;
	for (i = 0; i <= F_FLAGMAX; ++i)
	{
		if (!(symConfigEnt->txtTblBits & (PrvSymbolConvert(i))))
			continue;
	
		if (CtlGetValue(ckP[ckn]))
			symSetRange.dwFlags |= (symFlagTbl[i].flagBit);
		else
			symSetRange.dwFlags &= ~(symFlagTbl[i].flagBit);
		ckn++;
	}
	
	//Send the Command to ScanCmd Library
	ScnCmdWriteSymbologyConfig(symConfigEnt->symID, &symSetRange);
}	


/***********************************************************************
 *
 * FUNCTION: 		SymbologyConfigSetControls
 *
 * DESCRIPTION: 	Read the Symbology setting from ScanCmd Library
 *					and update the UI Menu
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void SymbologyConfigUpdateControls(int sel)
{
	FormType *frmP = FrmGetActiveForm();
	unsigned i, ckn;//, ndx;
	struct symConfigTblEnt *symConfigEnt;
	SymFlagsRange_t symSetRange;
	Boolean bEnabled = false;
	UInt32 testbit=0;
	ControlType *ckP[8];
	ControlPtr pCtl;
	ckP[0] = (ControlPtr) GetObjectPtr ( SymbologyCheck0Checkbox );
	ckP[1] = (ControlPtr) GetObjectPtr ( SymbologyCheck1Checkbox );
	ckP[2] = (ControlPtr) GetObjectPtr ( SymbologyCheck2Checkbox );
	ckP[3] = (ControlPtr) GetObjectPtr ( SymbologyCheck3Checkbox );
	ckP[4] = (ControlPtr) GetObjectPtr ( SymbologyCheck4Checkbox );
	ckP[5] = (ControlPtr) GetObjectPtr ( SymbologyCheck5Checkbox );
	ckP[6] = (ControlPtr) GetObjectPtr ( SymbologyCheck6Checkbox );
	ckP[7] = (ControlPtr) GetObjectPtr ( SymbologyCheck7Checkbox );

	//Set the EnableAll CheckBox
	//pCtl = (ControlPtr) GetObjectPtr( SymbologyEnableAllCheckbox );
	//CtlSetValue(pCtl, g_bEnabledAll);			
		
	for (i=0; i<8; ++i)
		CtlHideControl(ckP[i]);

	if (sel == noListSelection)
	{
		ListPtr pList = (ListPtr) GetObjectPtr( SymbologyCodeList );
		LstSetSelection(pList, noListSelection);
		return;
	}	
	
	//Read the Symbology setting for current selection
	symConfigEnt = &symConfigTbl[sel];
	if (symConfigEnt->txtTblBits & F_MINMAX)
		symSetRange.dwStructSize = sizeof(SymFlagsRange_t);
	else
		symSetRange.dwStructSize = sizeof(SymFlagsOnly_t);

	symSetRange.dwMask = SYM_MASK_ALL;
	ScnCmdReadSymbologyConfig(QT_CURRENT, symConfigEnt->symID, &symSetRange);
	
	if (symSetRange.dwFlags & SYMBOLOGY_ENABLE)
		bEnabled = true;

	pCtl = (ControlPtr) GetObjectPtr( SymbologyCodeEnableCheckbox );
	CtlSetValue(pCtl, bEnabled);	

	ckn=0;
	for (i=0; i<=F_FLAGMAX; ++i)
	{
		if (!(symConfigEnt->txtTblBits & (PrvSymbolConvert(i))))
			continue;

		CtlSetLabel(ckP[ckn], symFlagTbl[i].label);
		CtlSetValue(ckP[ckn], (symSetRange.dwFlags & (symFlagTbl[i].flagBit)) ? 1 : 0);
		CtlShowControl(ckP[ckn]);
		ckn++;
	}

	
	if (symConfigEnt->txtTblBits & F_MINMAX)
	{
		FieldAttrType attr, *attrP;
		FieldType *fldP;
			
		Char chrMinLen[8];
		Char chrMaxLen[8];
		
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMinFldField );
		attrP = &attr;
		FldGetAttributes (fldP, attrP);
		attrP->editable = true;
		FldSetAttributes (fldP, attrP);
		
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMaxFldField );
		attrP = &attr;
		FldGetAttributes (fldP, attrP);
		attrP->editable = true;
		FldSetAttributes (fldP, attrP);

		MemSet(chrMinLen, sizeof(chrMinLen), 0);
		MemSet(chrMaxLen, sizeof(chrMaxLen), 0);
		
		StrIToA( chrMinLen, symSetRange.dwMinLen );
		StrIToA( chrMaxLen, symSetRange.dwMaxLen );
				
		SetFieldText( SymbologySymbologyConfigFormMinFldField, chrMinLen, 7, false);  
		SetFieldText( SymbologySymbologyConfigFormMaxFldField, chrMaxLen, 7, false);  
	}
	else
	{
		FieldAttrType attr, *attrP;
		FieldType *fldP;
		
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMinFldField );
		attrP = &attr;
		FldGetAttributes (fldP, attrP);
		attrP->editable = false;
		FldSetAttributes (fldP, attrP);
		
		fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMaxFldField );
		attrP = &attr;
		FldGetAttributes (fldP, attrP);
		attrP->editable = false;
		FldSetAttributes (fldP, attrP);
		
		
		SetFieldText( SymbologySymbologyConfigFormMinFldField, "N.A.", 7, false);  
		SetFieldText( SymbologySymbologyConfigFormMaxFldField, "N.A.", 7, false); 
	}

	FrmDrawForm( frmP );
}



/***********************************************************************
 *
 * FUNCTION: 		SymbologyConfigFormInit
 *
 * DESCRIPTION: 	Init the Symbology Config Form
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void SymbologyConfigFormInit(FormType *frmP)
{
	FieldType *fldP;
	FieldAttrType attr, *attrP;
	MemHandle mem;
	
	attrP = &attr;
	
	fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMinFldField );
	mem = MemHandleNew(8);
	FldSetTextHandle(fldP, mem);
	FldGetAttributes (fldP, attrP);
	attrP->editable = false;
	FldSetAttributes (fldP, attrP);

	fldP = (FieldPtr) GetObjectPtr( SymbologySymbologyConfigFormMaxFldField );
	mem = MemHandleNew(8);
	FldSetTextHandle(fldP, mem);
	FldGetAttributes (fldP, attrP);
	attrP->editable = false;
	FldSetAttributes (fldP, attrP);
}


/***********************************************************************
 *
 * FUNCTION: 		SymbologyFormOnInit
 *
 * DESCRIPTION: 	Initializes controls on the Code Format Setup form by 
 * 					using the Scan Mgr API to get current decoder values.
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void SymbologyFormOnInit()
{
	FormPtr pForm = FrmGetActiveForm();
	FrmDrawForm( pForm );	
	SymbologyConfigFormInit( pForm );	
	SymbologyConfigUpdateControls(noListSelection);
}



/***********************************************************************
 *
 * FUNCTION: 		SymbologyConfigSetupHandleEvent
 *
 * DESCRIPTION: 	Event handler for the Code Format Setup form.
 * 					Handles the following events: 
 * 						frmOpenEvent and frmCloseEvent, 
 * 						ctlSelectEvent (for the OK and Cancel), 
 * 
 * PARAMETERS: 		ev - pointer to the event information structure
 * 
 * RETURNED: 		true  - we handled the event
 * 					false - not handled here, so pass it to next handler
 *
 ***********************************************************************/
Boolean SymbologyConfigSetupHandleEvent( EventPtr ev )
{
	Boolean bHandled = false;
	
	switch( ev->eType )
	{
		case lstSelectEvent:
			if( ev->data.ctlSelect.controlID == SymbologyCodeList)
			{	
				ListPtr pList = (ListPtr) GetObjectPtr(SymbologyCodeList);
				g_nSel = LstGetSelection( pList );
				SymbologyConfigUpdateControls(g_nSel);
				bHandled = true;	
			}
			break;
			
		case fldChangedEvent:
			if( ev->data.ctlSelect.controlID == SymbologySymbologyConfigFormMinFldField ||
				ev->data.ctlSelect.controlID == SymbologySymbologyConfigFormMaxFldField)
			
			{
				SymbologyConfigSetControls(g_nSel);
				bHandled = true;
			}
			break;
		
		case ctlSelectEvent:
			if( ev->data.ctlSelect.controlID == SymbologyCancelButton )
			{
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			else if( ev->data.ctlSelect.controlID == SymbologyOKButton )
			{
				SymbologyConfigSetControls(g_nSel);
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			
			else if( ev->data.ctlSelect.controlID == SymbologyEnableAllButton)
			{
				//Enable all BarCode									
				ControlPtr pCtl;
				ScnSetBarcodeEnabled( barALL, true);					
				ScnCmdSendParams(No_Beep);
				pCtl = (ControlPtr) GetObjectPtr( SymbologyCodeEnableCheckbox );
				CtlSetValue(pCtl, false);	
				g_nSel = noListSelection;
				SymbologyConfigUpdateControls(g_nSel);
				bHandled = true;	
			}
			else if( ev->data.ctlSelect.controlID == SymbologyDisableAllButton)
			{
				//Disable all BarCode	
				ControlPtr pCtl;								
				ScnSetBarcodeEnabled( barALL, false);				
				ScnCmdSendParams(No_Beep);
				pCtl = (ControlPtr) GetObjectPtr( SymbologyCodeEnableCheckbox );
				CtlSetValue(pCtl, false);	
				g_nSel = noListSelection;	
				SymbologyConfigUpdateControls(g_nSel);			
				bHandled = true;	
			}			

			else if( ev->data.ctlSelect.controlID == SymbologyCodeEnableCheckbox )
			{	
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(SymbologyCodeEnableCheckbox);
				ScnSetBarcodeEnabled(PrvBarCodeLstConversion(g_nSel), CtlGetValue(pCtl));	
				ScnCmdSendParams( No_Beep);			
		
				//Update the UI
				SymbologyConfigUpdateControls(g_nSel);
				bHandled = true;	
						
			}			
			
			else if( ev->data.ctlSelect.controlID == SymbologyCheck0Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck1Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck2Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck3Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck4Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck5Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck6Checkbox ||
			ev->data.ctlSelect.controlID == SymbologyCheck7Checkbox					
			)
			{	
				ControlPtr pCtl;
				pCtl = (ControlPtr) GetObjectPtr(SymbologyCodeEnableCheckbox);

				//Send the setting to ScanCmd Library
				SymbologyConfigSetControls(g_nSel);			
				bHandled = true;	
			}
			break;

		case frmOpenEvent:
			SymbologyFormOnInit();
			bHandled = true;
			break;
		
		case frmCloseEvent:
			break;
			
		default:
			break;
	}
	
	return bHandled;
}


//This function will convert the list number to SMLLib BarCode number 
tagBarType PrvBarCodeLstConversion(UInt32 LstNumber)
{
	switch(LstNumber)
	{
		case Codabar:
			return barCODABAR;
		case Code39:
			return barCODE39;
		case Int2of5:
			return barI2OF5;
		case Code93:
			return barCODE93;	
		case Code2of5:			
			return	barD25;		
		case IATA2of5:
			return barIATA25;
		case Matrix2of5:
			return barMATRIX25;
		case Code11:
			return barCODE11;
		case Code128:
			return barCODE128;		
		case Telepen:
			return barTELEPEN;
		case UPC_A:
			return barUPCA;	
		case CouponCode:
			return barCOUPON;
		case UPCE0:
			return barUPCE;	
		case UPCE1:
			return barUPCE1;	
		case EANJAN13:
			return barEAN13;			
		case EANJAN8:
			return barEAN8;
		case MSI:
			return barMSI;	
		case Plessey:
			return barPlessey;	
		case RSS14:
			return barRSS;
		case RSSLtd:
			return barRSS_LIM;		
		case RSSExp:
			return barRSS_EXP;
		case PosiCode:
			return barPOSICODE;	
		case TriopticCode:
			return barTRIOPTIC39;
		case CodablockF:
			return barCODABLOCK;
		case Code16K:
			return barCODE16K;
		case Code49:
			return barCODE49;
		case PDF417:
			return barPDF417;
		case MicroPDF417:
			return barMICROPDF;
		case Composite:
			return barCOMPOSITE;
		case TLC39:
			return barTLCODE39;
		case USPS4CB:
			return barUSPS4CB;
		case IDTAG:
			return barIDTAG;
		case Postnet:
			return barPOSTNET;
		case PlanetCode:
			return barPLANET;
		case BritishPost:
			return barBPO;
		case CanadaPost:
			return barCANPOST;
		case KixPost:
			return barDUTCHPOST;
		case AusPost:
			return barAUSPOST;
		case JapanPost:
			return barJAPOST;
		case ChinaPost:
			return barCHINAPOST;
		case KoreaPost:
			return barKOREAPOST;
		case QRCode:
			return barQR;
		case DataMatrix:
			return barDATAMAXTRIX;
		case MaxiCode:
			return barMAXICODE;
		case AztecCode:
			return barAZTEC;
	}
	//should not be here. //added to prevent warning msg.
	return barCODABAR;
}


/***********************************************************************
 ***********************************************************************
   OCR Configuration Dialog
 ***********************************************************************
 ***********************************************************************/
 
/***********************************************************************
 *
 * FUNCTION: 		OnOCRSetup
 *
 * DESCRIPTION: 	Makes OCRForm the active form
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
void OnOCRSetup()
{
	FrmGotoForm(OCRForm);
}

/***********************************************************************
 *
 * FUNCTION: 		OCROnInit
 *
 * DESCRIPTION: 	Initialize the OCR form
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void OCROnInit( void )
{
	FormPtr pForm = FrmGetActiveForm();
	FrmDrawForm(pForm);		
}

/***********************************************************************
 *
 * FUNCTION: 		OCRConfigFormClose
 *
 * DESCRIPTION: 	Close the OCR Config Form; Release Handles
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void OCRConfigFormClose()
{	
	MemHandle mem;
	FieldType *fldP;

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormTemplateField );
	mem = FldGetTextHandle(fldP);
	FldSetTextHandle(fldP, NULL);
	MemHandleFree(mem);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormGSetField );
	mem = FldGetTextHandle(fldP);
	FldSetTextHandle(fldP, NULL);
	MemHandleFree(mem);	

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormHSeField );
	mem = FldGetTextHandle(fldP);
	FldSetTextHandle(fldP, NULL);
	MemHandleFree(mem);	

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormCheckField );
	mem = FldGetTextHandle(fldP);
	FldSetTextHandle(fldP, NULL);
	MemHandleFree(mem);	
}

/***********************************************************************
 *
 * FUNCTION: 		OCRConfigFormInit
 *
 * DESCRIPTION: 	Initialize the OCR Config Form
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void OCRConfigFormInit()
{
	FieldType *fldP;
	MemHandle memTemplate, memGSet, memHSet, memCheck;
	char *pTemplate, *pGSet, *pHSet, *pCheck;
	OCRMode_t font;
	FormType *frmP = FrmGetActiveForm();

	memTemplate = MemHandleNew(256);
	memGSet = MemHandleNew(256);
	memHSet = MemHandleNew(256);
	memCheck = MemHandleNew(256);
	
	if (!memTemplate || !memGSet || !memHSet || !memCheck ) {
		MemHandleFree(memTemplate);
		MemHandleFree(memGSet);
		MemHandleFree(memHSet);
		MemHandleFree(memCheck);
		return;
	}

	pTemplate = (char *) MemHandleLock(memTemplate);
	pGSet = (char *) MemHandleLock(memGSet);
	pHSet = (char *) MemHandleLock(memHSet);
	pCheck = (char *) MemHandleLock(memCheck);
	
	//Get the OCR Config from the ScanCmd Library
	ScnCmdGetSetupOCR(QT_CURRENT, (UInt8*) &font, pTemplate, pGSet, pHSet, pCheck);
	
	MemHandleUnlock(memTemplate);
	MemHandleUnlock(memGSet);
	MemHandleUnlock(memHSet);
	MemHandleUnlock(memCheck);

	ListPtr pList = (ListPtr) GetObjectPtr( OCROCREnableList );
	LstSetSelection(pList, font);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormTemplateField );
	FldSetTextHandle(fldP, memTemplate);
	FldRecalculateField(fldP, true);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormGSetField );
	FldSetTextHandle(fldP, memGSet);
	FldRecalculateField(fldP, true);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormHSeField );
	FldSetTextHandle(fldP, memHSet);
	FldRecalculateField(fldP, true);
	
	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormCheckField );
	FldSetTextHandle(fldP, memCheck);
	FldRecalculateField(fldP, true);	
	FrmDrawForm( frmP );
}

/***********************************************************************
 *
 * FUNCTION: 		OCRConfigFormSaveConfig
 *
 * DESCRIPTION: 	Save the OCR Config Form
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void OCRConfigFormSaveConfig()
{
	FieldType *fldP;
	ListType *listP;
	char *pTemplate, *pGSet, *pHSet, *pCheck;
	UInt16 font;

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormTemplateField );
	pTemplate = FldGetTextPtr(fldP);
	
	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormGSetField );
	pGSet = FldGetTextPtr(fldP);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormHSeField );
	pHSet = FldGetTextPtr(fldP);

	fldP = (FieldPtr) GetObjectPtr( OCROCRConfigFormCheckField );
	pCheck = FldGetTextPtr(fldP);
	
	listP = (ListType*) GetObjectPtr(OCROCREnableList);
	font = LstGetSelection(listP);
	
	//Send the configuration to the ScanCmd Library
	ScnCmdSetupOCR(font, pTemplate, pGSet, pHSet, pCheck);
}



/***********************************************************************
 *
 * FUNCTION: 		OCRSetupHandleEvent
 *
 * DESCRIPTION: 	Event handler for the Beep Frequency Setup form.
 * 					Handles the following events: 
 * 						frmOpenEvent and frmCloseEvent
 * 						ctlSelectEvent (for OK, Cancel, and Test buttons) 
 * 
 * PARAMETERS: 		ev - pointer to the event information structure
 * 
 * RETURNED: 		true  - we handled the event
 * 					false - not handled here, so pass it to next handler
 *
 ***********************************************************************/
Boolean OCRSetupHandleEvent( EventPtr ev )
{
	Boolean bHandled = false;
	
	switch( ev->eType )
	{
	
		case ctlSelectEvent:
	
			if( ev->data.ctlSelect.controlID == OCRCancelButton )
			{
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			else if( ev->data.ctlSelect.controlID == OCROKButton )
			{	
				OCRConfigFormSaveConfig();			
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			break;
			
		case frmOpenEvent:
			OCROnInit();
			OCRConfigFormInit();
			bHandled = true;
			break;
			
		
		case frmCloseEvent:
			OCRConfigFormClose();
			break;
			
		default:
			break;
	}
	
	return bHandled;
}

/***********************************************************************
 ***********************************************************************
   About Dialog
 ***********************************************************************
 ***********************************************************************/
 
/***********************************************************************
 *
 * FUNCTION: 		OnAbout
 *
 * DESCRIPTION: 	Makes AboutForm the active form
 *
 * PARAMETERS: 		None.
 *
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
void OnAbout()
{
	FrmGotoForm( AboutForm );
}

/***********************************************************************
 *
 * FUNCTION: 		AboutOnInit
 *
 * DESCRIPTION: 	Initializes the controls on the About form using 
 * 					the Scan Manager API to get current decoder values.
 * 
 * PARAMETERS: 		None.
 * 
 * RETURNED: 		Nothing.
 *
 ***********************************************************************/
static void AboutOnInit( void )
{
	Char szText[MAX_PACKET_LENGTH];
	
	Coord x, y;
	FormPtr pForm = FrmGetActiveForm();

	// Draw the form - then draw the version information (labels used as place holders only)
	FrmDrawForm( pForm );	

	// Scan Manager Version
	FrmGetObjectPosition( pForm, FrmGetObjectIndex( pForm, AboutScanMgrVerLabel ), &x, &y );
	ScnGetScanManagerVersion( szText, MAX_PACKET_LENGTH );
	WinDrawChars( szText, StrLen(szText), x, y );

	// Scan Port Driver Version
	FrmGetObjectPosition( pForm, FrmGetObjectIndex( pForm, AboutScanDrvVerLabel ), &x, &y );
	ScnGetScanPortDriverVersion( szText, MAX_PACKET_LENGTH );
	WinDrawChars( szText, StrLen(szText), x, y );

	// Decoder Firmware Version
	FrmGetObjectPosition( pForm, FrmGetObjectIndex( pForm, AboutDecoderVerLabel ), &x, &y );
	ScnGetDecoderVersion( szText, MAX_PACKET_LENGTH );
	WinDrawChars( 	szText, StrLen(szText), x, y );

}

/***********************************************************************
 *
 * FUNCTION: 		AboutHandleEvent
 *
 * DESCRIPTION: 	Event handler for the About form.
 * 					Handles the following events: 
 * 						frmOpenEvent and frmCloseEvent, 
 * 						ctlSelectEvent (for the OK button) 
  * 
 * PARAMETERS: 		ev - pointer to the event information structure
 * 
 * RETURNED: 		true  - we handled the event
 * 					false - not handled here, so pass it to next handler
 *
 ***********************************************************************/
Boolean AboutHandleEvent( EventPtr ev )
{
	Boolean bHandled = false;
	
	switch( ev->eType )
	{
		case ctlSelectEvent:
			if( ev->data.ctlSelect.controlID == AboutOKButton )
			{
				FrmGotoForm(MainForm);
				bHandled = true;
			}
			break;

		case frmOpenEvent:
			AboutOnInit();
			bHandled = true;
			break;
			
		case frmCloseEvent:
			break;
			
		default:
			break;
	}
	
	return bHandled;
}


/***********************************************************************
 *
 * FUNCTION: 		HwrUtilsLib_OpenLibrary
 *
 * DESCRIPTION: 	User-level call to open the library.  This inline function
 * 					handles the messy task of finding or loading the library
 * 					and calling its open function, including handling cleanup
 * 					if the library could not be opened.
 * 
 * PARAMETERS: 		refNumP
 *					Pointer to UInt16 variable that will hold the new
 *      			library reference number for use in later calls
 *
 * 					clientContextPointer to variable for returning client context.  
 *					The client context is used to maintain client-specific data
 *					for multiple client support. The value returned here will
 *		 			be used as a parameter for other library functions which
 *		 			require a client context.  
 * 
 * RETURNED: 		errNone
 *					memErrNotEnoughSpace
 *     			 	sysErrLibNotFound
 *      			sysErrNoFreeRAM
 *      			sysErrNoFreeLibSlots
 ***********************************************************************/
static Err HwrUtilsLib_OpenLibrary(UInt16 *refNumP)
{
	Err error;
	Boolean loaded = false;
	
	// First try to find the library
	// If not found, load the library instead
	if (SysLibFind(kHwrUtilsLibName, refNumP) == sysErrLibNotFound){
		error = SysLibLoad(kHwrUtilsLibType, kHwrUtilsLibCreator, refNumP);
		loaded = true;
	}
	
	if (error == errNone){
		error = HwrUtilsLibOpen(*refNumP);
		if (error != errNone){
			if (loaded)
				SysLibRemove(*refNumP);
			*refNumP = sysInvalidRefNum;
		}
	}
	return error;
}


/***********************************************************************
 *
 * FUNCTION: 		HwrUtilsLib_CloseLibrary
 *
 * DESCRIPTION: 	User-level call to closes the shared library.  This handles 
 * 					removal of the library from system if there are no users
 * 					remaining.
 * 
 * PARAMETERS: 		refNum
 *					Library reference number obtained from ImgLib_OpenLibrary().
 *
 * 					clientContext
 *					client context (as returned by the open call)
 *
 * 
 * RETURNED: 		errNone
 *					sysErrParamErr
 *
 ***********************************************************************/
static Err HwrUtilsLib_CloseLibrary(UInt16 refNum)
{
	Err error;
	if (refNum == sysInvalidRefNum)
		return sysErrParamErr;

	error = HwrUtilsLibClose(refNum);

	// no users left, so unload library
	if (error == errNone)
		SysLibRemove(refNum);
		
	return error;
}




