/******************************************************************************
 *
 * 	Barcode Scanner Demo Application
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			Utils.c
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Commonly used utility functions.
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 

#ifdef __cplusplus
extern "C" {
#endif

#include "PalmOS.h"
#include "Utils.h"
#include "ScanDemoRsc.h"
#include "ScanMgrDef.h"


#ifdef __cplusplus
}
#endif

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This function return the object pointer
 *
 * PARAMETERS:  ResourceId - resource Id
 *
 * RETURNED:    Control Pointer
 *
 ***********************************************************************/
MemPtr GetObjectPtr(UInt16 nResourceId)
{
	FormPtr frm = FrmGetActiveForm();
	if(frm)
		return (FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, nResourceId)));
	return NULL;
}


/***********************************************************************
 *
 * FUNCTION:    SetFieldText
 *
 * DESCRIPTION:	Draw Text to a field object
 *              
 *
 * PARAMETERS: 	nTextFieldID - Field Id
 *				pText - Text Pointer
 *				nMaxSize - Text Size
 *				bRedraw  - Redrawn required
 *
 * RETURNED:   	None
 *
 ***********************************************************************/
void SetFieldText(UInt16 nTextFieldID, const Char *pText, Int16 nSize, Boolean bRedrawn)
{
	MemHandle hFieldText;
	Char *   pFieldText;
	FieldPtr pField;

	pField = (FieldPtr)GetObjectPtr(nTextFieldID);
	if(!pField)
		return;
		
	hFieldText = FldGetTextHandle( pField );
	if( !hFieldText )
		hFieldText = MemHandleNew( nSize );
	else
	{
		UInt32 curSize = MemHandleSize( hFieldText );
		if(curSize < nSize)
			MemHandleResize(hFieldText, nSize) ;
	}

	if(hFieldText)
	{
		Int16 len = StrLen(pText);
		
		pFieldText = (Char * )MemHandleLock( hFieldText );
		
		if (len > nSize)
		{
			StrNCopy( pFieldText, pText, nSize-1);
			pFieldText[nSize-1] = '\0';
		}
		else
			StrCopy( pFieldText, pText );
			
		MemHandleUnlock( hFieldText );

 		FldSetTextHandle( pField, (MemHandle)hFieldText );
		FldSetTextAllocatedSize( pField, nSize );
		
		FldSetMaxChars( pField, nSize-1);

		FldRecalculateField( pField, true );
		if(bRedrawn)
			FldDrawField( pField );
	}
}


/***********************************************************************
 *
 * FUNCTION:    FreeFieldHandle
 *
 * DESCRIPTION: Free the text field
 *
 * PARAMETERS:  Field Id
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
void FreeFieldHandle(Int16 nFieldId)
{
	FieldPtr pField = (FieldPtr)GetObjectPtr(nFieldId);;
	MemHandle handle = FldGetTextHandle(pField);
	if (handle)
	{
		FldSetTextHandle(pField, 0);
		MemHandleFree(handle);
	}
}

/***********************************************************************
 *
 * FUNCTION:    ScnGetBarTypeStr
 *
 * DESCRIPTION: Get the string of specified barcode values
 *
 * PARAMETERS:  barType 	- BarCode Number
 *				pszBarType 	- Pointer to the string of barcode name
 *              nStrLen	 	- Max length of the barcode name
 *
 * RETURNED:    True if success; False if fail
 *
 ***********************************************************************/
Boolean ScnGetBarTypeStr(UInt8 nbarType, Char *  pszBarType, UInt16 nStrLen)
{
	Boolean bResult = true;
	switch (nbarType)
	{
		case BCTYPE_NOT_APPLICABLE: // 0x00
			SysCopyStringResource( pszBarType, BCTypeNotApplicableString);
			break;	
		case BCTYPE_CODE39: // 0x01
			SysCopyStringResource( pszBarType, BCTypeCode39String);
			break;	
		case BCTYPE_CODABAR: // 0x02
			SysCopyStringResource( pszBarType, BCTypeCodabarString);
			break;	
		case BCTYPE_CODE128: // 0x03
			SysCopyStringResource( pszBarType, BCTypeCode128String);
			break;	
		case BCTYPE_D2OF5: // 0x04
			SysCopyStringResource( pszBarType, BCTypeDiscrete2Of5String);
			break;	
		case BCTYPE_IATA2OF5: // 0x05
			SysCopyStringResource( pszBarType, BCTypeIATA2of5String);
			break;	
		case BCTYPE_I2OF5: // 0x06
			SysCopyStringResource( pszBarType, BCTypeInterleaved2Of5String);
			break;	
		case BCTYPE_CODE93: // 0x07
			SysCopyStringResource( pszBarType, BCTypeCode93String);
			break;	
		case BCTYPE_UPCA: // 0x08
			SysCopyStringResource( pszBarType, BCTypeUPCAString);
			break;	
		case BCTYPE_UPCE0: // 0x09
			SysCopyStringResource( pszBarType, BCTypeUPCEOString);
			break;	
		case BCTYPE_EAN8: // 0x0A
			SysCopyStringResource( pszBarType, BCTypeEAN8String);
			break;	
		case BCTYPE_EAN13: // 0x0B
			SysCopyStringResource( pszBarType, BCTypeEAN13String);
			break;	
		case BCTYPE_MSI_PLESSEY: // 0x0E
			SysCopyStringResource( pszBarType, BCTypeMSIPlesseyString);
			break;	
		case BCTYPE_EAN128: // 0x0F
			SysCopyStringResource( pszBarType, BCTypeEAN128String);
			break;	
		case BCTYPE_UPCE1:  // 0x10
			SysCopyStringResource( pszBarType, BCTypeUPCE1String);
			break;	
		case BCTYPE_PDF417: // 0x11
			SysCopyStringResource( pszBarType, BCTypePDF417String);
			break;	
		case BCTYPE_CODE39_FULL_ASCII: // 0x13
			SysCopyStringResource( pszBarType, BCTypeTLC39StringString);
			break;	
		case BCTYPE_TRIOPTIC_CODE39: // 0x15
			SysCopyStringResource( pszBarType, BCTypeTriopticCode39String);
			break;	
		case BCTYPE_BOOKLAND_EAN: // 0x16
			SysCopyStringResource( pszBarType, BCTypeBooklandEANString);
			break;	
		case BCTYPE_COUPON_CODE: // 0x17
			SysCopyStringResource( pszBarType, BCTypeCouponCodeString);
			break;	
		case BCTYPE_UPCA_2SUPPLEMENTALS: // 0x48
			SysCopyStringResource( pszBarType, BCTypeUPCA2SuppsString);
			break;	
		case BCTYPE_UPCE0_2SUPPLEMENTALS: // 0x49
			SysCopyStringResource( pszBarType, BCTypeUPCEO2SuppsString);
			break;	
		case BCTYPE_EAN8_2SUPPLEMENTALS:  // 0x4A
			SysCopyStringResource( pszBarType, BCTypeEAN82SuppsString);
			break;	
		case BCTYPE_EAN13_2SUPPLEMENTALS: // 0x4B
			SysCopyStringResource( pszBarType, BCTypeEAN132SuppsString);
			break;	
		case BCTYPE_UPCE1_2SUPPLEMENTALS: // 0x50
			SysCopyStringResource( pszBarType, BCTypeUPCE12SuppsString);
			break;	
		case BCTYPE_EAN8_5SUPPLEMENTALS:  // 0x8A
			SysCopyStringResource( pszBarType, BCTypeEAN85SuppsString);
			break;	
		case BCTYPE_EAN13_5SUPPLEMENTALS: // 0x8B
			SysCopyStringResource( pszBarType, BCTypeEAN135SuppsString);
			break;	
		case BCTYPE_UPCA_5SUPPLEMENTALS:  // 0x88
			SysCopyStringResource( pszBarType, BCTypeUPCA5SuppsString);
			break;	
		case BCTYPE_UPCE0_5SUPPLEMENTALS: // 0x89
			SysCopyStringResource( pszBarType, BCTypeUPCEO5SuppsString);
			break;	
		case BCTYPE_UPCE1_5SUPPLEMENTALS: // 0x90
			SysCopyStringResource( pszBarType, BCTypeUPCE15SuppsString);
			break;	
		
		//Add for Code32
		case BCTYPE_CODE32:
			SysCopyStringResource( pszBarType, BCTypeCODE32String);
			break;
	
		//Extended Support  (0x31-0x3F, 0x60-0x6D)
		 case BCTYPE_AZTEC:
			SysCopyStringResource( pszBarType, BCTypeAZTECString);
			break;
	 	 case BCTYPE_CODE11:
			SysCopyStringResource( pszBarType, BCTypeCODE11String);
			break;
		 case BCTYPE_CODE49:
			SysCopyStringResource( pszBarType, BCTypeCODE49String);
			break;		 
		 case BCTYPE_COMPOSITE:
			SysCopyStringResource( pszBarType, BCTypeCOMPOSITEString);
			break;		 
		 case BCTYPE_DATAMATRIX:
			SysCopyStringResource( pszBarType, BCTypeDATAMATRIXString);
			break;		 
		 case BCTYPE_MAXICODE:
			SysCopyStringResource( pszBarType, BCTypeMAXICODEString);
			break;		 
		 case BCTYPE_MICROPDF:
			SysCopyStringResource( pszBarType, BCTypeMICROPDFString);
			break;		 
		 case BCTYPE_POSTNET: 
			SysCopyStringResource( pszBarType, BCTypePOSTNETString);
			break;		 
		 case BCTYPE_OCR:     
			SysCopyStringResource( pszBarType, BCTypeOCRString);
			break;		 
		 case BCTYPE_QR:      
			SysCopyStringResource( pszBarType, BCTypeQRString);
			break;		 
		 case BCTYPE_RSS:
			SysCopyStringResource( pszBarType, BCTypeRSSString);
			break;		      
		 case BCTYPE_ISBT:    
			SysCopyStringResource( pszBarType, BCTypeISBTString);
			break;		 
		 case BCTYPE_BPO:     
			SysCopyStringResource( pszBarType, BCTypeBPOString);
			break;		 
		 case BCTYPE_CANPOST: 
			SysCopyStringResource( pszBarType, BCTypeCANPOSTString);
			break;		 
		 case BCTYPE_AUSPOST: 
			SysCopyStringResource( pszBarType, BCTypeAUSPOSTString);
			break;		 
		 case BCTYPE_IATA25:  
			SysCopyStringResource( pszBarType, BCTypeIATA25String);
			break;		 
		 case BCTYPE_CODABLOCK:    	
			SysCopyStringResource( pszBarType, BCTypeCODABLOCKString);
			break;		 
		 case BCTYPE_JAPOST:       	
			SysCopyStringResource( pszBarType, BCTypeJAPOSTString);
			break;		 
		 case BCTYPE_PLANET:       	
			SysCopyStringResource( pszBarType, BCTypePLANETString);
			break;		 
		 case BCTYPE_DUTCHPOST:   
			SysCopyStringResource( pszBarType, BCTypeDUTCHPOSTString);
			break;		   	
		 case BCTYPE_MATRIX25:	    
			SysCopyStringResource( pszBarType, BCTypeMATRIX25String);
			break;		 
		 case BCTYPE_CHINAPOST:		
			SysCopyStringResource( pszBarType, BCTypeCHINAPOSTString);
			break;		 
		 case BCTYPE_KOREAPOST:	
			SysCopyStringResource( pszBarType, BCTypeKOREAPOSTString);
			break;		 
		 case BCTYPE_TELEPEN:	
			SysCopyStringResource( pszBarType, BCTypeTELEPENString);
			break;		 
		 case BCTYPE_CODE16K:	
			SysCopyStringResource( pszBarType, BCTypeCODE16KString);
			break;		 
		 case BCTYPE_POSICODE:	
			SysCopyStringResource( pszBarType, BCTypePOSICODEString);
			break;		 
		 case BCTYPE_COUPONCODE:	
			SysCopyStringResource( pszBarType, BCTypeCOUPONCODEString);
			break;		 
		 case BCTYPE_USPS4CB:    
			SysCopyStringResource( pszBarType, BCTypeUSPS4CBString);
			break;		 
		 case BCTYPE_IDTAG:	
			SysCopyStringResource( pszBarType, BCTypeIDTAGString);
			break;		 
		
		default: 
			StrCopy(pszBarType,"UNKNOWN");	
			bResult = false;
			break;
	}		
	return bResult;
}

